#!/usr/bin/env node
;
var GitHubApi, _, argv, async, filename, fs, github, note, path, projectCase, quiet, split, token, yargs;

fs = require('fs');

path = require('path');

_ = require('lodash');

yargs = require('yargs');

GitHubApi = require('github');

async = require('async');

split = require('split');

argv = yargs.usage('Usage: $0 -t [token]').demand('t').alias('t', 'token').describe('t', 'OAuth token').alias('f', 'file').describe('f', 'todo.txt file')["default"]('f', path.join(process.env.HOME, "Dropbox", "todo", "todo.txt")).alias('q', 'quiet').describe('q', 'Minimize console output').env('GITHUB_TODOTXT').alias('c', 'config').describe('c', 'Config file')["default"]('c', path.join(process.env.HOME, ".github-todotxt.json")).config('config').help('h').alias('h', 'help').argv;

projectCase = function(str) {
  return _.upperFirst(_.camelCase(str));
};

token = argv.t;

filename = argv.f;

quiet = argv.q != null;

note = function(str) {
  if (!quiet) {
    return process.stdout.write(str);
  }
};

github = new GitHubApi({
  debug: false
});

github.authenticate({
  type: "oauth",
  token: token
});

async.parallel([
  function(callback) {
    var todos;
    todos = [];
    return fs.createReadStream(filename).pipe(split()).on('data', function(line) {
      var m, todo;
      if (line.match(/\S/)) {
        m = line.match(/issue:(\S+)/);
        todo = {
          text: line
        };
        if (m) {
          todo.issue = m[1];
        }
        return todos.push(todo);
      }
    }).on('error', function(err) {
      return callback(err, null);
    }).on('end', function() {
      return callback(null, todos);
    });
  }, function(callback) {
    var getIssues;
    getIssues = function(page, acc, callback) {
      var props;
      note(".");
      props = {
        state: "all",
        filter: "assigned",
        per_page: 100,
        page: page
      };
      return github.issues.getAll(props, function(err, issues) {
        if (err) {
          return callback(err);
        } else {
          acc = _.concat(acc, issues);
          if (issues.length >= 100) {
            return getIssues(page + 1, acc, callback);
          } else {
            note("\n");
            return callback(null, acc);
          }
        }
      });
    };
    if (!quiet) {
      note("Getting issues...");
    }
    return getIssues(1, [], callback);
  }
], function(err, results) {
  var backup, i, id, issue, issues, len, line, number, project, repo, title, todo, todos, ts;
  if (err) {
    return console.error(err);
  } else {
    todos = results[0], issues = results[1];
    if (!quiet) {
      note(todos.length + " lines in " + filename + "\n");
      note(issues.length + " issues on Github\n");
    }
    for (i = 0, len = issues.length; i < len; i++) {
      issue = issues[i];
      repo = issue.repository.full_name;
      number = issue.number;
      id = repo + "#" + number;
      todo = _.find(todos, {
        issue: id
      });
      if (todo != null) {
        if (todo.text.match(/^x/)) {
          if (issue.state === "open") {
            "not closing issue";
          }
        } else {
          if (issue.state === "closed") {
            note("Marking line for issue " + id + " complete.\n");
            todo.text = "x " + todo.text;
          }
        }
      } else if (issue.state === "open") {
        note("Adding line for issue " + id + ".\n");
        ts = issue.created_at.substr(0, 10);
        project = projectCase(repo.split("/")[1]);
        title = issue.title;
        line = ts + " " + title + " issue:" + repo + "#" + number + " +" + project;
        if (issue.milestone != null) {
          line += " +" + (projectCase(issue.milestone.title));
        }
        todos.push({
          text: line
        });
      }
    }
    backup = filename + ".bak";
    return async.waterfall([
      function(callback) {
        return fs.rename(filename, backup, callback);
      }, function(callback) {
        var data, texts;
        texts = _.map(todos, "text");
        data = texts.join("\n") + "\n";
        return fs.writeFile(filename, data, callback);
      }
    ], function(err) {
      if (err) {
        return console.error(err);
      }
    });
  }
});
